local GlobalAddonName, ExRT = ...

ExRT.lib = {}

function ExRT.lib.AddShadowComment(self,hide,moduleName,userComment,userFontSize,userOutline)
	if self.moduleNameString then
		if hide then
			self.moduleNameString:Hide()
		else
			local selfWidth = self:GetWidth()
			local selfHeight = self:GetHeight()
			self.moduleNameString:SetSize(selfWidth,selfHeight)
			self.moduleNameString:Show()
		end
	elseif not hide and moduleName then
		local selfWidth = self:GetWidth()
		local selfHeight = self:GetHeight()
		self.moduleNameString = ExRT.lib.CreateText(self,selfWidth,selfHeight,"BOTTOMRIGHT", -5, 4,"RIGHT","BOTTOM",ExRT.mds.defFont, 18,moduleName or "",nil)
		self.moduleNameString:SetTextColor(1, 1, 1, 0.8)
	end

	if self.userCommentString then
		if hide then
			self.userCommentString:Hide()
		else
			local selfWidth = self:GetWidth()
			local selfHeight = self:GetHeight()
			self.userCommentString:SetSize(selfWidth,selfHeight)
			self.userCommentString:Show()
		end
	elseif not hide and userComment then
		local selfWidth = self:GetWidth()
		local selfHeight = self:GetHeight()
		self.userCommentString = ExRT.lib.CreateText(self,selfWidth,selfHeight,"BOTTOMRIGHT", -5, 20,"RIGHT","BOTTOM",ExRT.mds.defFont, userFontSize or 18,userComment or "",nil,0,0,0,nil,userOutline)
		self.userCommentString:SetTextColor(0, 0, 0, 0.7)
	end
end

function ExRT.lib.CreateSlider(name,parent,width,height,x,y,minVal,maxVal,text,defVal,relativePoint)
	defVal = defVal or maxVal

	local self = CreateFrame("Slider",name,parent,"OptionsSliderTemplate")
	self:SetWidth(width)
	self:SetHeight(height)
	self:SetPoint(relativePoint or "TOPLEFT",parent, x, y)
	self.textLow = _G[name.."Low"]
	self.textHigh = _G[name.."High"]
	self.text = _G[name.."Text"]
	self:SetOrientation("HORIZONTAL")
	self:SetMinMaxValues(minVal, maxVal)
	self.minValue, self.maxValue = self:GetMinMaxValues() 
	self.textLow:SetText(self.minValue)
	self.textHigh:SetText(self.maxValue)
	self.text:SetText(text)
	self.tooltipText = defVal
	self:SetValueStep(1)
	self:SetValue(defVal)

	self:SetScript("OnMouseWheel", function(self,delta) 
		if tonumber(self:GetValue()) == nil then return end
		self:SetValue(tonumber(self:GetValue())+delta)
	end)

	function self:tooltipShow(t)
		GameTooltip:SetOwner(t, "ANCHOR_RIGHT")
		GameTooltip:SetText(t.tooltipText or "", nil, nil, nil, nil, true)
		GameTooltip:Show()
	end
	function self:tooltipHide()
		GameTooltip_Hide()
	end
	function self:tooltipReload(self)
		if GameTooltip:IsVisible() then
			self:tooltipHide()
			self:tooltipShow(self)
		end
	end
	self:SetScript("OnEnter", function(t) self:tooltipShow(t) end)
	self:SetScript("OnLeave", self.tooltipHide)

	return self
end

function ExRT.lib.CreateScrollBar(name,parent,width,height,x,y,minVal,maxVal,relativePoint,isHoriz)
	local self = CreateFrame("Slider", name, parent)
	self.bg = self:CreateTexture(nil, "BACKGROUND")
	self.bg:SetAllPoints(true)
	self.bg:SetTexture(0, 0, 0, 0.5)
	self.thumb = self:CreateTexture(nil, "OVERLAY")
	self.thumb:SetTexture("Interface\\Buttons\\UI-ScrollBar-Knob")
	self.thumb:SetSize(25, 25)
	self:SetThumbTexture(self.thumb)
	self:SetOrientation(isHoriz and "HORIZONTAL" or "VERTICAL")
	self:SetSize(width, height)
	self:SetPoint(relativePoint or "TOPLEFT", parent, x, y)
	self:SetMinMaxValues(minVal, maxVal)
	self:SetValue(minVal)

	return self
end

function ExRT.lib.CreateScrollBar2(name,parent,width,height,x,y,minVal,maxVal,relativePoint)
	local self = CreateFrame("Slider", name, parent)
	self.bg = self:CreateTexture(nil, "BACKGROUND")
	self.bg:SetAllPoints(true)
	self.bg:SetTexture(0, 0, 0, 0.5)
	self.thumb = self:CreateTexture(nil, "OVERLAY")
	self.thumb:SetTexture("Interface\\Buttons\\UI-ScrollBar-Knob")
	self.thumb:SetSize(25, 25)
	self:SetThumbTexture(self.thumb)
	self:SetOrientation(isHoriz and "HORIZONTAL" or "VERTICAL")
	self:SetSize(width, height - 32)
	self:SetPoint(relativePoint or "TOPLEFT", parent, x, y - 16)
	self:SetMinMaxValues(minVal, maxVal)
	self:SetValue(minVal)
	
	self.buttonUP = CreateFrame("Button",nil,self,"UIPanelScrollUPButtonTemplate")
	self.buttonUP:SetSize(16,16)
	self.buttonUP:SetPoint("BOTTOM",self,"TOP",0,0) 
	self.buttonUP:SetScript("OnClick",function()
		local min_,max_ = self:GetMinMaxValues()
		local val = self:GetValue()
		if (val - 1) < min_ then
			self:SetValue(min_)
		else
			self:SetValue(val - 1)
		end
	end)

	self.buttonDown = CreateFrame("Button",nil,self,"UIPanelScrollDownButtonTemplate")
	self.buttonDown:SetPoint("TOP",self,"BOTTOM",0,0) 
	self.buttonDown:SetSize(16,16)
	self.buttonDown:SetScript("OnClick",function()
		local min_,max_ = self:GetMinMaxValues()
		local val = self:GetValue()
		if (val + 1) > max_ then
			self:SetValue(max_)
		else
			self:SetValue(val + 1)
		end
	end)
	
	function self:buttonsState(UP,DOWN)
		self.buttonUP:SetEnabled(UP)
		self.buttonDown:SetEnabled(DOWN)
	end
	function self:reButtonsState()
		local event = ExRT.mds.Round(self:GetValue())
		local min_,max_ = self:GetMinMaxValues()
		if max_ == min_ then
			self:buttonsState(nil,nil)
		elseif event == min_ then
			self:buttonsState(nil,true)
		elseif event == max_ then
			self:buttonsState(true,nil)
		else
			self:buttonsState(true,true)
		end
	end
	
	return self
end

function ExRT.lib.OnEnterTooltip(self,anchorUser)
	GameTooltip:SetOwner(self,anchorUser or "ANCHOR_RIGHT")
	GameTooltip:SetText(self.tooltipText or "", nil, nil, nil, nil, true)
	GameTooltip:Show()
end

function ExRT.lib.OnLeaveTooltip(self)
	GameTooltip_Hide()
end

function ExRT.lib.TooltipShow(self,anchorUser,title,...)
	if title then
		local x,y=0,0
		if type(anchorUser) == "table" then
			x = anchorUser[2]
			y = anchorUser[3]
			anchorUser = anchorUser[1] or "ANCHOR_RIGHT"
		elseif not anchorUser then
			anchorUser = "ANCHOR_RIGHT"
		end
		GameTooltip:SetOwner(self,anchorUser or "ANCHOR_RIGHT",x,y)
		GameTooltip:SetText(title)
		for i=1,select("#", ...) do
			local line = select(i, ...)
			if type(line) == "table" then
				GameTooltip:AddLine(unpack(line))
			else
				GameTooltip:AddLine(line)
			end
		end
		GameTooltip:Show()
	end
end

function ExRT.lib.TooltipHide()
	GameTooltip_Hide()
end

function ExRT.lib.OnEnterHyperLinkTooltip(self,data)
	if not data then return end
	local x = self:GetRight()
	if ( x >= ( GetScreenWidth() / 2 ) ) then
		GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	else
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	end
	GameTooltip:SetHyperlink(data)
	GameTooltip:Show()
end

function ExRT.lib.OnLeaveHyperLinkTooltip(self)
	GameTooltip_Hide()
end

function ExRT.lib.ShowOrHide(self,bool)
	if not self then return end
	if bool then
		self:Show()
	else
		self:Hide()
	end
end

function ExRT.lib.SetAlphas(alpha,...)
	for i=1,select("#", ...) do
		local self = select(i, ...)
		self:SetAlpha(alpha)
	end
end

function ExRT.lib.CreateTabFrame(name,parent,width,height,x,y,tabNum,activeTabNum,...)
	local self = CreateFrame("Frame",name,parent)
	self:SetSize(width,height)
	self:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", edgeFile = "Interface/Tooltips/UI-Tooltip-Border",tile = true, tileSize = 16, edgeSize = 16, insets = { left = 5, right = 5, top = 5, bottom = 5 }})
	self:SetBackdropColor(0,0,0,0.5)
	self:SetPoint("TOPLEFT",x,y)

	self.tabs = {}
	for i=1,tabNum do
		self.tabs[i] = CreateFrame("Frame",format("%sFrame%d",name,i),self)
		self.tabs[i]:SetSize(width,height)
		self.tabs[i]:SetPoint("TOPLEFT", 0,0)

		self.tabs[i].button = CreateFrame("Button", format("%sTab%d",name,i), self, "OptionsFrameTabButtonTemplate")
		self.tabs[i].button:SetText(select(i, ...))
		PanelTemplates_TabResize(self.tabs[i].button, 0, nil, nil, self.tabs[i].button:GetFontString():GetStringWidth(), self.tabs[i].button:GetFontString():GetStringWidth())
		self.tabs[i].button.id = i
		self.tabs[i].button:SetScript("OnClick", function(s)
			self.selected = s.id
			self:UpdateTabs()
		end)

		if i == 1 then
			self.tabs[i].button:SetPoint("TOPLEFT", 10, 24)
		else
			self.tabs[i].button:SetPoint("LEFT", self.tabs[i-1].button, "RIGHT", -16, 0)
			self.tabs[i]:Hide()
		end
		PanelTemplates_DeselectTab(self.tabs[i].button)
	end
	PanelTemplates_SelectTab(self.tabs[activeTabNum or 1].button)

	self.tabCount = tabNum
	self.selected = activeTabNum or 1
	function self:UpdateTabs()
		for i=1,self.tabCount do
			if i == self.selected then
				PanelTemplates_SelectTab(self.tabs[i].button)
				self.tabs[i]:Show()
			else
				PanelTemplates_DeselectTab(self.tabs[i].button)
				self.tabs[i]:Hide()
			end
		end	  
	end

	return self
end

function ExRT.lib.CreateText(parent,width,height,relativePoint,x,y,hor,ver,font,fontSize,text,tem,colR,colG,colB,shadow,outline)
	if not tem and not font then 
		tem = "GameFontNormal" 
	end
	if outline then 
		outline = "OUTLINE" 
	end
	local self = parent:CreateFontString(nil,"ARTWORK",tem)
	if not tem then
		self:SetFont(font, fontSize, outline)
	elseif fontSize then
		local filename = self:GetFont()
		self:SetFont(filename,fontSize, outline)
	end
	self:SetSize(width,height)
	self:SetPoint(relativePoint or "TOPLEFT", x,y)
	self:SetJustifyH(hor or "LEFT")
	self:SetJustifyV(ver or "MIDDLE")
	if colR and colG and colB then
		self:SetTextColor(colR,colG,colB,1)
	end
	if shadow then
		self:SetShadowOffset(1,-1)
	end
	if text then
		self:SetText(text)
	end
	return self
end

function ExRT.lib.CreateEditBox(name,parent,width,height,relativePoint,x,y,tooltip,maxLetters,onlyNum,template,defText)
	local self = CreateFrame("EditBox",name,parent,template)
	self:SetSize(width,height)
	self:SetPoint(relativePoint or "TOPLEFT",x,y)
	if not template then
		local GameFontNormal_Font = GameFontNormal:GetFont()
		self:SetFont(GameFontNormal_Font,12)
		self:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",edgeFile = "Interface\\AddOns\\ExRT\\media\\border.tga",edgeSize = 8,tileSize = 0,insets = {left = 2.5,right = 2.5,top = 2.5,bottom = 2.5}})
		self:SetBackdropColor(0, 0, 0, 0.8) 
		self:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
		self:SetTextInsets(10,10,0,0)
	end
	if defText then
		self:SetText(defText)
		self:SetCursorPosition(0)
	end
	self:SetAutoFocus( false )
	if tooltip then
		self:SetScript("OnEnter",function(s) ExRT.lib.OnEnterTooltip(s) end)
		self:SetScript("OnLeave",function() ExRT.lib.OnLeaveTooltip() end)
		self.tooltipText = tooltip
	end
	if maxLetters then
		self:SetMaxLetters(maxLetters)
	end
	if onlyNum then
		self:SetNumeric(true)
	end
	self:SetScript("OnEscapePressed",function(this) 
		this:ClearFocus()
	end)
	return self
end

function ExRT.lib.CreateScrollFrame(name,parent,width,height,relativePoint,x,y,verticalHeight)
	local self = CreateFrame("ScrollFrame", name, parent)
	self:SetSize(width,height)
	self:SetPoint(relativePoint or "TOPLEFT",x,y)
	
	self.backdrop = CreateFrame("Frame", nil, self)
	self.backdrop:SetPoint("TOPLEFT",self,-5,5)
	self.backdrop:SetSize(width+10,height+10)
	self.backdrop:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", edgeFile = "Interface/Tooltips/UI-Tooltip-Border",tile = true, tileSize = 16, edgeSize = 16, insets = { left = 5, right = 5, top = 5, bottom = 5 }})
	self.backdrop:SetBackdropColor(0,0,0,0)
	
	self.content = CreateFrame("Frame", nil, self) 
	self.content:SetSize(width-16, verticalHeight) 
	self:SetScrollChild(self.content)
	
	self.C = self.content
	
	self.ScrollBar = ExRT.lib.CreateScrollBar2(name.."ScrollBar",self,16,height,0,0,0,max(verticalHeight-height,0),"TOPRIGHT")
	self.ScrollBar:SetScript("OnValueChanged",function (this,value)
		self:SetVerticalScroll(value) 
		this:reButtonsState()
	end)
	self.ScrollBar.reButtonsState(self.ScrollBar)
	
	self:SetScript("OnMouseWheel", function(self,delta) 
		delta = delta * 20
		local min_,max_ = self.ScrollBar:GetMinMaxValues()
		local val = self.ScrollBar:GetValue()
		if (val - delta) < min_ then
			self.ScrollBar:SetValue(min_)
		elseif (val - delta) > max_ then
			self.ScrollBar:SetValue(max_)
		else
			self.ScrollBar:SetValue(val - delta)
		end
	end)
	
	return self
end


function ExRT.lib.CreateMultiEditBox(name,parent,width,height,relativePoint,x,y)
	local self=CreateFrame("ScrollFrame", name, parent, "InputScrollFrameTemplate")
	self:SetSize(width,height)
	self:SetFrameStrata("FULLSCREEN")
	self:SetPoint(relativePoint or "TOPLEFT",x,y)
	self.FocusButton:SetFrameLevel(9)
	self.EditBox:SetFrameLevel(10)
	self.ScrollBar:SetFrameLevel(11)
	self.EditBox:SetFontObject("ChatFontNormal")
	self.EditBox:SetMaxLetters(0)
	self.CharCount:Hide()
	self.EditBox:SetWidth(width - 30)
	return self
end

function ExRT.lib.CreateMultiEditBox2(name,parent,width,height,relativePoint,x,y)
	local self=ExRT.lib.CreateMultiEditBox(name,parent,width,height,relativePoint,x,y)
	self.EditBox:SetScript("OnHide",function(slf)
		slf:SetText("")
	end)
	
	self:SetScript("OnVerticalScroll",nil)
	self:SetScript("OnScrollRangeChanged",nil)
	self:SetScript("OnMouseWheel",function (slf,delta)
		local v = self.ScrollBar:GetValue() - delta
		local v_min,v_max = self.ScrollBar:GetMinMaxValues()
		if v < v_min then 
			v = v_min 
		elseif v > v_max then
			v = v_max
		end
		self.ScrollBar:SetValue(v)
	end)
	self.EditBox:SetScript("OnUpdate",nil)
	self.EditBox:SetScript("OnTextChanged",nil)
	self.ScrollBar:Show()
	self.ScrollBar:SetMinMaxValues(0,0.01)
	self.ScrollBar:SetValue(0.01)
	
	return self
end

function ExRT.lib.CreateEditSliderBox(parent,width,height,x,y,txt)
	if width <= height*2 then return end
	local self = CreateFrame("Frame",nil,parent)
	self:SetSize(width-height*2,height)
	self:SetPoint("TOPLEFT",x+height,y)
	self:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",edgeFile = "Interface\\AddOns\\ExRT\\media\\border.tga",edgeSize = 8,tileSize = 0,insets = {left = 2.5,right = 2.5,top = 2.5,bottom = 2.5}})
	self:SetBackdropColor(0, 0, 0, 0.8) 
	self:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)

	self:SetScript("OnEnter",function(s) ExRT.lib.OnEnterTooltip(s) end)
	self:SetScript("OnLeave",function() ExRT.lib.OnLeaveTooltip() end)

	self.selected = 1

	self.t = self:CreateFontString(nil,"ARTWORK")
	local GameFontNormal_Font = GameFontNormal:GetFont()
	self.t:SetFont(GameFontNormal_Font,12)
	self.t:SetAllPoints()
	self.t:SetJustifyH("CENTER")
	self.t:SetJustifyV("MIDDLE")
	self.t:SetText(txt or "")

	self.l = {}
	for i=1,2 do
		self.l[i] = CreateFrame("Button",nil,self)
		self.l[i]:SetSize(height,height)
		self.l[i]:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",edgeFile = "Interface\\AddOns\\ExRT\\media\\border.tga",edgeSize = 8,tileSize = 0,insets = {left = 2.5,right = 2.5,top = 2.5,bottom = 2.5}})
		self.l[i]:SetBackdropColor(0, 0, 0, 0.8) 
		self.l[i]:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)

		self.l[i].t = self.l[i]:CreateFontString(nil,"ARTWORK")
		self.l[i].t:SetFont(GameFontNormal_Font,12)
		self.l[i].t:SetAllPoints()
		self.l[i].t:SetJustifyH("CENTER")
		self.l[i].t:SetJustifyV("MIDDLE")
	end
	self.l[1]:SetPoint("LEFT",-height,0)
	self.l[1].t:SetText("<")

	self.l[2]:SetPoint("RIGHT",height,0)
	self.l[2].t:SetText(">")

	return self
end

function ExRT.lib.CreateButton(name,parent,width,height,relativePoint,x,y,text,isDisabled,tooltip)
	local self = CreateFrame("Button",nil,parent,"UIPanelButtonTemplate")
	self:SetSize(width,height)
	if x and y then
		self:SetPoint(relativePoint or "TOPLEFT",x,y) 
	end
	self:SetText(text) 
	if isDisabled then
		self:Disable()
	end
	if tooltip then
		self.tooltip = text
		if text == "" then
			self.tooltip = " "
		end
		self.tooltipText = tooltip
		self:SetScript("OnEnter",function(this)
			ExRT.lib.TooltipShow(this,"ANCHOR_TOP",self.tooltip,{self.tooltipText,1,1,1,true}) 
		end)
		self:SetScript("OnLeave",function() 
			ExRT.lib.TooltipHide()
		end)
	end
	return self
end

function ExRT.lib.CreateIcon(name,parent,size,relativePoint,x,y,textureIcon,isButton)
	local self = CreateFrame(isButton and "Button" or "FRAME",name,parent)
	self:SetSize(size,size)
	self:SetPoint(relativePoint or "TOPLEFT", x,y)
	self.texture = self:CreateTexture(nil, "BACKGROUND")
	self.texture:SetTexture(textureIcon or "Interface\\Icons\\INV_MISC_QUESTIONMARK", true)
	self.texture:SetAllPoints()
	if isButton then
 		self:EnableMouse(true)
		self:RegisterForClicks("LeftButtonDown")
	end
	return self
end

function ExRT.lib.CreateCheckBox(name,parent,relativePoint,x,y,text,checked,tooltip,textLeft)
	local self = CreateFrame("CheckButton",name,parent,"UICheckButtonTemplate")  
	if x and y then
		self:SetPoint(relativePoint or "TOPLEFT",x,y)
	end
	self.text:SetText(text)
	self:SetChecked(checked)
	self.tooltip = text
	if tooltip then
		if text == "" then
			self.tooltip = " "
		end
		self.tooltipText = tooltip
	end
	self:SetScript("OnEnter",function(this)
		ExRT.lib.TooltipShow(this,"ANCHOR_TOP",self.tooltip,{self.tooltipText,1,1,1,true}) 
	end)
	self:SetScript("OnLeave",function() 
		ExRT.lib.TooltipHide()
	end)
	self:SetScript("OnClick", function(self,event) 
		if self:GetChecked() then
			self:On()
		else
			self:Off()
		end
	end)
	if textLeft then
		self.text:ClearAllPoints()
		self.text:SetPoint("RIGHT",self,"LEFT",-2,0)
	end
	
	return self
end

function ExRT.lib.CreateHoverHighlight(parent)
	parent.hl = parent:CreateTexture(nil, "BACKGROUND")
	parent.hl:SetPoint("TOPLEFT", 0, 0)
	parent.hl:SetPoint("BOTTOMRIGHT", 0, 0)
	parent.hl:SetTexture("Interface\\QuestFrame\\UI-QuestLogTitleHighlight",true)
	parent.hl:SetBlendMode("ADD")
	parent.hl:Hide()
end

function ExRT.lib.CreateBackTextureForDebug(parent)
	local self = parent:CreateTexture(nil, "BACKGROUND")
	self:SetTexture(1, 0, 0, 0.3)
	self:SetAllPoints()
end

function ExRT.lib.CreateHelpButton(parent,helpPlateArray,isTab)
	local self = CreateFrame("Button",nil,parent,"MainHelpPlateButton")	-- После использования кнопки не дает юзать спелл дизенчант. лень искать решение, не юзайте кнопку часто
	self:SetPoint("CENTER",parent,"TOPLEFT",0,0) 
	self:SetScale(0.8)
	self:SetScript("OnClick",function()
		local helpPlate
		if isTab then
			helpPlate = helpPlateArray[isTab.selected]
		else
			helpPlate = helpPlateArray
		end
		if helpPlate and not HelpPlate_IsShowing(helpPlate) then
			HelpPlate_Show(helpPlate, parent, self, true)
		else
			HelpPlate_Hide(true)
		end
		if self.Click2 then
			self:Click2()
		end
	end)
	self:SetScript("OnHide",function()
		HelpPlate_Hide()
	end)
	return self
end

function ExRT.lib.CreateScrollList(name,parent,relativePoint,x,y,width,linesNum)
	local self = CreateFrame("Frame",name,parent)
	local height = linesNum * 16 + 8
	self:SetSize(width,height)
	self:SetPoint(relativePoint or "TOPLEFT",x,y)
	self:SetBackdrop({bgFile = "", edgeFile = "Interface/Tooltips/UI-Tooltip-Border",tile = true, tileSize = 16, edgeSize = 16, insets = { left = 5, right = 5, top = 5, bottom = 5 }})
	
	self.ScrollBar = ExRT.lib.CreateScrollBar2(name.."ScrollBar",self,16,height-8,-3,-4,1,10,"TOPRIGHT")
	
	self.List = {}
	for i=1,linesNum do
		self.List[i] = CreateFrame("Button",name.."List"..tostring(i),self)
		self.List[i]:SetSize(width - 22,16)
		self.List[i]:SetPoint("TOPLEFT",3,-(i-1)*16-4)
		
		self.List[i]:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestLogTitleHighlight","ADD")
		
		self.List[i].text = ExRT.lib.CreateText(self.List[i],width - 30,16,nil,4,0,"LEFT","MIDDLE",nil,12,"List"..tostring(i),nil,1,1,1,1)
		
		self.List[i].PushedTexture = self.List[i]:CreateTexture()
		self.List[i].PushedTexture:SetTexture("Interface\\QuestFrame\\UI-QuestLogTitleHighlight")
		self.List[i].PushedTexture:SetBlendMode("ADD")
		self.List[i].PushedTexture:SetAllPoints()
		self.List[i].PushedTexture:SetVertexColor(1,1,0,1)
		
		self.List[i]:SetDisabledTexture(self.List[i].PushedTexture)
		
		self.List[i]:SetFontString(self.List[i].text)
		self.List[i]:SetPushedTextOffset(2, -1)
		
		self.List[i]:SetScript("OnClick",function (this)
			if not self.dontDisable then
				for j=1,linesNum do
					if j ~= i then
						self.List[j]:SetEnabled(true)
					else
						self.List[j]:SetEnabled(nil)
					end
				end
			end
			self.selected = this.index
			self:SetListValue(this.index)
		end)
		
		self.List[i]:SetScript("OnEnter",function (this)
			if self.HoverListValue then
				self:HoverListValue(true,this.index)
			end
		end)
		self.List[i]:SetScript("OnLeave",function (this)
			if self.HoverListValue then
				self:HoverListValue(false,this.index)
			end
		end)		
	end
	
	self.L = {}
	function self.Update()
		local val = ExRT.mds.Round(self.ScrollBar:GetValue())
		local j = 0
		for i=val,#self.L do
			j = j + 1
			self.List[j]:SetText(self.L[i])
			if not self.dontDisable then
				if i ~= self.selected then
					self.List[j]:SetEnabled(true)
				else
					self.List[j]:SetEnabled(nil)
				end
			end
			self.List[j]:Show()
			self.List[j].index = i
			if j >= linesNum then
				break
			end
		end
		for i=(j+1),linesNum do
			self.List[i]:Hide()
		end
		self.ScrollBar:SetMinMaxValues(1,max(#self.L-linesNum+1,1))
		self.ScrollBar:reButtonsState()
	end
	
	self:SetScript("OnMouseWheel",function(this, delta)
		if delta > 0 then
			self.ScrollBar.buttonUP:Click("LeftButton")
		else
			self.ScrollBar.buttonDown:Click("LeftButton")
		end
	end)
	
	self:SetScript("OnShow",self.Update)
	self.ScrollBar:SetScript("OnValueChanged",self.Update)
	
	return self
end

function ExRT.lib.CreateScrollCheckList(name,parent,relativePoint,x,y,width,linesNum)
	local self = CreateFrame("Frame",name,parent)
	local height = linesNum * 16 + 8
	self:SetSize(width,height)
	self:SetPoint(relativePoint or "TOPLEFT",x,y)
	self:SetBackdrop({bgFile = "", edgeFile = "Interface/Tooltips/UI-Tooltip-Border",tile = true, tileSize = 16, edgeSize = 16, insets = { left = 5, right = 5, top = 5, bottom = 5 }})
	
	self.ScrollBar = ExRT.lib.CreateScrollBar2(name.."ScrollBar",self,16,height-8,-3,-4,1,10,"TOPRIGHT")
	
	self.List = {}
	for i=1,linesNum do
		self.List[i] = CreateFrame("Button",name.."List"..tostring(i),self)
		self.List[i]:SetSize(width - 22,16)
		self.List[i]:SetPoint("TOPLEFT",3,-(i-1)*16-4)
		
		self.List[i]:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestLogTitleHighlight","ADD")
		
		self.List[i].text = ExRT.lib.CreateText(self.List[i],width - 50,16,nil,24,0,"LEFT","MIDDLE",nil,12,"List"..tostring(i),nil,1,1,1,1)
		
		self.List[i].chk = CreateFrame("CheckButton",nil,self.List[i],"UICheckButtonTemplate")  
		self.List[i].chk:SetScale(0.75)
		self.List[i].chk:SetPoint("TOPLEFT",0,5)
		self.List[i].chk:SetScript("OnClick", function(this,event) 
			if this:GetChecked() then
				self.C[this:GetParent().index] = true
			else
				self.C[this:GetParent().index] = nil
			end
			self.ValueChanged(self)
		end)
		
		self.List[i].PushedTexture = self.List[i]:CreateTexture()
		self.List[i].PushedTexture:SetTexture("Interface\\QuestFrame\\UI-QuestLogTitleHighlight")
		self.List[i].PushedTexture:SetBlendMode("ADD")
		self.List[i].PushedTexture:SetAllPoints()
		self.List[i].PushedTexture:SetVertexColor(1,1,0,1)
		
		self.List[i]:SetDisabledTexture(self.List[i].PushedTexture)
		
		self.List[i]:SetFontString(self.List[i].text)
		self.List[i]:SetPushedTextOffset(2, -1)
		
		self.List[i]:SetScript("OnClick",function (this)
			self.C[this.index] = not self.C[this.index]
			self.List[i].chk:SetChecked(self.C[this.index])
			
			self.ValueChanged(self)
		end)
		
		self.List[i]:SetScript("OnEnter",function (this)
			if self.HoverListValue then
				self:HoverListValue(true,this.index)
			end
		end)
		self.List[i]:SetScript("OnLeave",function (this)
			if self.HoverListValue then
				self:HoverListValue(false,this.index)
			end
		end)		
	end
	
	self.L = {}
	self.C = {}
	function self.Update()
		local val = ExRT.mds.Round(self.ScrollBar:GetValue())
		local j = 0
		for i=val,#self.L do
			j = j + 1
			self.List[j]:SetText(self.L[i])
			self.List[j].chk:SetChecked(self.C[i])
			self.List[j]:Show()
			self.List[j].index = i
			if j >= linesNum then
				break
			end
		end
		for i=(j+1),linesNum do
			self.List[i]:Hide()
		end
		self.ScrollBar:SetMinMaxValues(1,max(#self.L-linesNum+1,1))
		self.ScrollBar:reButtonsState()
	end
	
	self:SetScript("OnMouseWheel",function(this, delta)
		if delta > 0 then
			self.ScrollBar.buttonUP:Click("LeftButton")
		else
			self.ScrollBar.buttonDown:Click("LeftButton")
		end
	end)
	
	self:SetScript("OnShow",self.Update)
	self.ScrollBar:SetScript("OnValueChanged",self.Update)
	
	return self
end

function ExRT.lib.CreatePopupFrame(name,width,height,title)
	local self = CreateFrame("Frame",name,UIParent,"UIPanelDialogTemplate")
	self:SetSize(width,height)
	self:SetPoint("CENTER")
	self:SetFrameStrata("DIALOG")
	self:SetClampedToScreen(true)
	self:EnableMouse(true)
	self:SetMovable(true)
	self:RegisterForDrag("LeftButton")
	self:SetDontSavePosition(true)
	self:SetScript("OnDragStart", function(self) 
		self:StartMoving() 
	end)
	self:SetScript("OnDragStop", function(self) 
		self:StopMovingOrSizing() 
	end)
	self:Hide()
	
	if title then
		self.title = ExRT.lib.CreateText(self,width,14,"TOPLEFT",20,-8,"LEFT","TOP",nil,14,title,nil,1,1,1,1)
	end
	
	return self
end

function ExRT.lib.CreateOneTab(parent,width,height,relativePoint,x,y,name)
	local self = CreateFrame("Frame",nil,parent)
	self:SetSize(width,height)
	self:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", edgeFile = "Interface/Tooltips/UI-Tooltip-Border",tile = true, tileSize = 16, edgeSize = 16, insets = { left = 5, right = 5, top = 5, bottom = 5 }})
	self:SetBackdropColor(0,0,0,0.5)
	self:SetPoint(relativePoint or "TOPLEFT",x,y)
	if name then
		self.name = ExRT.lib.CreateText(self,width-20,20,"TOP",0,17,nil,nil,nil,nil,name,"GameFontNormal")
	end
	return self
end

function ExRT.lib.CreateColorPickButton(parent,width,height,relativePoint,x,y,cR,cG,cB,cA)
	local self = CreateFrame("Button",nil,parent)
	self:SetPoint(relativePoint or "TOPLEFT",x,y)
	self:SetSize(width,height)
	self:SetBackdrop({edgeFile = "Interface\\AddOns\\ExRT\\media\\border.tga", edgeSize = 8})
	
	self:SetScript("OnEnter",function ()
		self:SetBackdropBorderColor(0.5,1,0,5,1)
	end)
	self:SetScript("OnLeave",function ()
	  	self:SetBackdropBorderColor(1,1,1,1)
	end)
	
	self.color = self:CreateTexture(nil, "BACKGROUND")
	self.color:SetTexture(cR or 0, cG or 0, cB or 0, cA or 1)
	self.color:SetAllPoints()
	
	return self
end

function ExRT.lib.CreateScrollTabsFrame(name,parent,relativePoint,x,y,width,height,noSelfBorder,...)
	local linesNum = floor((height - 8)/16)
	height = height - ((height-8)%16) + (noSelfBorder and 0 or 10)
	relativePoint = relativePoint or "TOPLEFT"
	
	local self
	if noSelfBorder then
		self = CreateFrame("Frame",name,parent)
		self:SetSize(width,height)
		self:SetPoint(relativePoint,x,y)
	else
		self = ExRT.lib.CreateOneTab(parent,width,height,relativePoint,x,y)
		_G[name] = self
	end
	self.list = ExRT.lib.CreateScrollList(name.."ScrollList",self,"TOPLEFT",noSelfBorder and 0 or 5,noSelfBorder and 0 or -5,200,linesNum)
	self.tab = {}
	self.listCount = select("#", ...)
	for i=1,self.listCount do
		self.list.L[i] = select(i, ...)
		self.tab[i] = ExRT.lib.CreateOneTab(self,width-205-(noSelfBorder and 0 or 10),height-(noSelfBorder and 0 or 10),"TOPLEFT",205+(noSelfBorder and 0 or 5),noSelfBorder and 0 or -5)
	end
	self.list:Update()
	
	function self.list:SetListValue(index)
		for i=1,_G[name].listCount do
			ExRT.lib.ShowOrHide(_G[name].tab[i],i == index)
		end
	end
	self.list.selected = 1
	self.list:SetListValue(1)

	return self
end

function ExRT.lib.CreateHiddenFrame(name,parent,relativePoint,x,y,width,height)
	local self = CreateFrame("Frame",name,parent)
	self:SetSize(width,height)
	self:SetPoint(relativePoint or "TOPLEFT",x,y)
		
	return self
end

function ExRT.lib.CreateDropDown(name,parent,relativePoint,x,y,width,tooltip)
	local self = CreateFrame("Frame", name, parent, "UIDropDownMenuTemplate")
	self:SetPoint(relativePoint or "TOPLEFT",x,y)
	self:SetWidth(width)
	UIDropDownMenu_SetWidth(self, width)
	self.width = width
	
	return self
end
